@@ -4,15 +4,18 @@
     ~~~~~~~~~~~~~~~~~~~~~
 
 
-    :copyright: 2007 by Armin Ronacher.
+    :copyright: (c) 2009 by the Werkzeug Team, see AUTHORS for more details.
     :license: BSD license.
 """
-from py.test import raises
+from nose.tools import assert_raises
+
+from werkzeug.wrappers import Response
 from werkzeug.routing import Map, Rule, NotFound, BuildError, RequestRedirect
 from werkzeug.utils import create_environ
 
 
 def test_basic_routing():
+    """Basic URL routing"""
     map = Map([
         Rule('/', endpoint='index'),
         Rule('/foo', endpoint='foo'),
@@ -22,8 +25,8 @@ def test_basic_routing():
     assert adapter.match('/') == ('index', {})
     assert adapter.match('/foo') == ('foo', {})
     assert adapter.match('/bar/') == ('bar', {})
-    raises(RequestRedirect, lambda: adapter.match('/bar'))
-    raises(NotFound, lambda: adapter.match('/blub'))
+    assert_raises(RequestRedirect, lambda: adapter.match('/bar'))
+    assert_raises(NotFound, lambda: adapter.match('/blub'))
 
 
 test_environ_defaults = '''
@@ -48,6 +51,7 @@ NotFound: 404 Not Found
 
 
 def test_basic_building():
+    """Basic URL building"""
     map = Map([
         Rule('/', endpoint='index'),
         Rule('/foo', endpoint='foo'),
@@ -66,10 +70,11 @@ def test_basic_building():
     assert adapter.build('barf', {'bazf': 0.815}) == 'http://example.org/bar/0.815'
     assert adapter.build('barp', {'bazp': 'la/di'}) == 'http://example.org/bar/la/di'
     assert adapter.build('blah', {}) == '/hehe'
-    raises(BuildError, lambda: adapter.build('urks'))
+    assert_raises(BuildError, lambda: adapter.build('urks'))
 
 
 def test_defaults():
+    """URL routing defaults"""
     map = Map([
         Rule('/foo/', defaults={'page': 1}, endpoint='foo'),
         Rule('/foo/<int:page>', endpoint='foo')
@@ -77,7 +82,7 @@ def test_defaults():
     adapter = map.bind('example.org', '/')
 
     assert adapter.match('/foo/') == ('foo', {'page': 1})
-    raises(RequestRedirect, lambda: adapter.match('/foo/1'))
+    assert_raises(RequestRedirect, lambda: adapter.match('/foo/1'))
     assert adapter.match('/foo/2') == ('foo', {'page': 2})
     assert adapter.build('foo', {}) == '/foo/'
     assert adapter.build('foo', {'page': 1}) == '/foo/'
@@ -85,6 +90,7 @@ def test_defaults():
 
 
 def test_greedy():
+    """URL routing greedy settings"""
     map = Map([
         Rule('/foo', endpoint='foo'),
         Rule('/<path:bar>', endpoint='bar'),
@@ -102,6 +108,7 @@ def test_greedy():
 
 
 def test_path():
+    """URL routing path converter behavior"""
     map = Map([
         Rule('/', defaults={'name': 'FrontPage'}, endpoint='page'),
         Rule('/Special', endpoint='special'),
@@ -118,7 +125,7 @@ def test_path():
     adapter = map.bind('example.org', '/')
 
     assert adapter.match('/') == ('page', {'name':'FrontPage'})
-    raises(RequestRedirect, lambda: adapter.match('/FrontPage'))
+    assert_raises(RequestRedirect, lambda: adapter.match('/FrontPage'))
     assert adapter.match('/Special') == ('special', {})
     assert adapter.match('/2007') == ('year', {'year':2007})
     assert adapter.match('/Some/Page') == ('page', {'name':'Some/Page'})
@@ -129,3 +136,58 @@ def test_path():
     assert adapter.match('/User:thomas') == ('user', {'username':'thomas'})
     assert adapter.match('/User:thomas/projects/werkzeug') == ('userpage', {'username':'thomas', 'name':'projects/werkzeug'})
     assert adapter.match('/Files/downloads/werkzeug/0.2.zip') == ('files', {'file':'downloads/werkzeug/0.2.zip'})
+
+
+def test_dispatch():
+    """URL routing dispatch helper"""
+    env = create_environ('/')
+    map = Map([
+        Rule('/', endpoint='root'),
+        Rule('/foo/', endpoint='foo')
+    ])
+    adapter = map.bind_to_environ(env)
+
+    raise_this = None
+    def view_func(endpoint, values):
+        if raise_this is not None:
+            raise raise_this
+        return Response(repr((endpoint, values)))
+    dispatch = lambda p, q=False: Response.force_type(adapter.dispatch(view_func, p,
+                                                      catch_http_exceptions=q), env)
+
+    assert dispatch('/').data == "('root', {})"
+    assert dispatch('/foo').status_code == 301
+    raise_this = NotFound()
+    assert_raises(NotFound, lambda: dispatch('/bar'))
+    assert dispatch('/bar', True).status_code == 404
+
+
+def test_http_host_before_server_name():
+    """URL routing HTTP host takes precedence before server name"""
+    env = {
+        'HTTP_HOST':            'wiki.example.com',
+        'SERVER_NAME':          'web0.example.com',
+        'SERVER_PORT':          '80',
+        'SCRIPT_NAME':          '',
+        'PATH_INFO':            '',
+        'REQUEST_METHOD':       'GET',
+        'wsgi.url_scheme':      'http'
+    }
+    map = Map([Rule('/', endpoint='index', subdomain='wiki')])
+    adapter = map.bind_to_environ(env, server_name='example.com')
+    assert adapter.match('/') == ('index', {})
+    assert adapter.build('index', force_external=True) == 'http://wiki.example.com/'
+    assert adapter.build('index') == '/'
+
+    env['HTTP_HOST'] = 'admin.example.com'
+    adapter = map.bind_to_environ(env, server_name='example.com')
+    assert adapter.build('index') == 'http://wiki.example.com/'
+
+
+def test_adapter_url_parameter_sorting():
+    """Optional adapter URL parameter sorting"""
+    map = Map([Rule('/', endpoint='index')], sort_parameters=True,
+              sort_key=lambda x: x[1])
+    adapter = map.bind('localhost', '/')
+    assert adapter.build('index', {'x': 20, 'y': 10, 'z': 30},
+        force_external=True) == 'http://localhost/?y=10&x=20&z=30'
